package cn.iocoder.yudao.module.reward.controller.app.redeem;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.reward.api.reward.RedeemRewardApi;
import cn.iocoder.yudao.module.reward.api.reward.dto.RedeemRewardRespDTO;
import cn.iocoder.yudao.module.reward.controller.app.redeem.vo.*;
import cn.iocoder.yudao.module.reward.service.redeem.RewardRedeemService;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageRespVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "用户 APP - 兑换记录管理")
@RequestMapping("/reward/redeem")
public class AppRedeemRewardController {
    @Resource
    private RedeemRewardApi redeemRewardApi;

    @Resource
    private RewardRedeemService rewardRedeemService;

    @PostMapping("record/list")
    @ApiOperation("兑换记录列表")
    //@PreAuthenticated
    public CommonResult<List<AppRewardRedeemListRespVO>> recordList(@Valid @RequestBody AppRewardRedeemReqVO request) {
        return success(rewardRedeemService.recordList(request.getMemberId()));
    }

    @PostMapping("record/detail")
    @ApiOperation("兑换详情")
    //@PreAuthenticated
    public CommonResult<AppRewardRedeemDetailRespVO> detail(@Valid @RequestBody AppRewardRedeemDetailReqVO request) {
        RewardRedeemPageRespVO detail = rewardRedeemService.detail(Long.valueOf(request.getRedeemId()));
        if (detail != null) {
            AppRewardRedeemDetailRespVO appRewardRedeemDetailRespVO = new AppRewardRedeemDetailRespVO();
            BeanUtils.copyProperties(detail, appRewardRedeemDetailRespVO);
            String status = appRewardRedeemDetailRespVO.getStatus().toString();
            appRewardRedeemDetailRespVO.setStatusZh(DictFrameworkUtils.getDictDataFromCache("reward_redeem_status", status).getLabel());
            appRewardRedeemDetailRespVO.setStatusEn(DictFrameworkUtils.getDictDataFromCache("reward_redeem_status", status).getLabelEn());
            appRewardRedeemDetailRespVO.setStatusFr(DictFrameworkUtils.getDictDataFromCache("reward_redeem_status", status).getLabelFr());
            String redeemType = appRewardRedeemDetailRespVO.getRedeemType().toString();
            appRewardRedeemDetailRespVO.setRedeemTypeZh(DictFrameworkUtils.getDictDataFromCache("way_of_receiving", redeemType).getLabel());
            appRewardRedeemDetailRespVO.setRedeemTypeEn(DictFrameworkUtils.getDictDataFromCache("way_of_receiving", redeemType).getLabelEn());
            appRewardRedeemDetailRespVO.setRedeemTypeFr(DictFrameworkUtils.getDictDataFromCache("way_of_receiving", redeemType).getLabelFr());
            return success(appRewardRedeemDetailRespVO);
        }
        return null;
    }

    @PostMapping("/single")
    @ApiOperation("兑换礼品")
    @Idempotent(timeout = 5)
    //@PreAuthenticated
    public CommonResult<Boolean> redeemReward(@Valid @RequestBody AppRedeemRewardReqVO redeemRewardReqVO) {
        // APP/WEB兑换设置兑换时间
        redeemRewardReqVO.setRedemptionTime(new Date());
        return success(redeemRewardApi.redeemReward(redeemRewardReqVO));
    }
    @PostMapping("/send-sms-code")
    @ApiOperation(value = "兑换礼品发送手机验证码")
    @OperateLog(enable = false) // 避免 Post 请求被记录操作日志
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> sendSmsCode(@RequestBody @Valid RedeemRewardRespDTO reqVO) {
        redeemRewardApi.sendSmsCode(reqVO);
        return success(true);
    }
}
